@@ -729,19 +729,18 @@ const LocalDocument = new Lang.Class({
             return;
 
         let file = Gio.file_new_for_uri(this.uri);
-        file.delete_async(GLib.PRIORITY_DEFAULT, null, Lang.bind(this,
+        file.trash_async(GLib.PRIORITY_DEFAULT, null, Lang.bind(this,
             function(source, res) {
                 try {
-                    file.delete_finish(res);
+                    file.trash_finish(res);
                 } catch(e) {
-                    log('Unable to delete ' + this.uri + ': ' + e.message);
+                    log('Unable to trash ' + this.uri + ': ' + e.message);
                 }
             }));
     }
 });
 
-const _GOOGLE_DOCS_SCHEME_LABELS = "http://schemas.google.com/g/2005/labels";
-const _GOOGLE_DOCS_TERM_STARRED = "http://schemas.google.com/g/2005/labels#starred";
+const GOOGLE_PREFIX = 'google:drive:';
 
 const GoogleDocument = new Lang.Class({
     Name: 'GoogleDocument',
@@ -762,10 +761,11 @@ const GoogleDocument = new Lang.Class({
 
         let authorizer = new GData.GoaAuthorizer({ goa_object: source.object });
         let service = new GData.DocumentsService({ authorizer: authorizer });
+        let gdata_id = this.identifier.substring(GOOGLE_PREFIX.length);
 
         service.query_single_entry_async
             (GData.DocumentsService.get_primary_authorization_domain(),
-             this.identifier, null,
+             gdata_id, null,
              GData.DocumentsText,
              cancellable, Lang.bind(this,
                  function(object, res) {
@@ -787,7 +787,7 @@ const GoogleDocument = new Lang.Class({
             function(entry, service, exception) {
                 if (exception) {
                     // try loading from the most recent cache, if any
-                    GdPrivate.pdf_loader_load_uri_async(this.identifier, cancellable, Lang.bind(this,
+                    GdPrivate.pdf_loader_load_uri_async(this.identifier, passwd, cancellable, Lang.bind(this,
                         function(source, res) {
                             try {
                                 let docModel = GdPrivate.pdf_loader_load_uri_finish(res);
@@ -1013,7 +1013,7 @@ const SkydriveDocument = new Lang.Class({
             function(entry, service, exception) {
                 if (exception) {
                     // try loading from the most recent cache, if any
-                    GdPrivate.pdf_loader_load_uri_async(this.identifier, cancellable, Lang.bind(this,
+                    GdPrivate.pdf_loader_load_uri_async(this.identifier, passwd, cancellable, Lang.bind(this,
                         function(source, res) {
                             try {
                                 let docModel = GdPrivate.pdf_loader_load_uri_finish(res);
@@ -1125,7 +1125,7 @@ const DocumentManager = new Lang.Class({
 
     _identifierIsGoogle: function(identifier) {
         return (identifier &&
-                (identifier.indexOf('https://docs.google.com') != -1));
+                (identifier.indexOf(GOOGLE_PREFIX) != -1));
     },
 
     _identifierIsOwncloud: function(identifier) {
