@@ -60,21 +60,6 @@ const Searchbar = new Lang.Class({
         item.add(this._searchContainer);
         toolbar.insert(item, 0);
 
-        this._searchEntry.connect('key-press-event', Lang.bind(this,
-            function(widget, event) {
-                let keyval = event.get_keyval()[1];
-
-                if (keyval == Gdk.KEY_Escape) {
-                    Application.application.change_action_state('search', GLib.Variant.new('b', false));
-                    return true;
-                } else if (keyval == Gdk.KEY_Return) {
-                    this.emit('activate-result');
-                    return true;
-                }
-
-                return false;
-            }));
-
         this._searchEntry.connect('search-changed', Lang.bind(this,
             function() {
                 if (this.searchChangeBlocked)
@@ -116,13 +101,16 @@ const Searchbar = new Lang.Class({
         this.widget.destroy();
     },
 
+    _isEscapeEvent: function(event) {
+        let keyval = event.get_keyval()[1];
+        return (keyval == Gdk.KEY_Escape);
+    },
+
     _isKeynavEvent: function(event) {
         let keyval = event.get_keyval()[1];
         let state = event.get_state()[1];
 
-        if (keyval == Gdk.KEY_Tab ||
-            keyval == Gdk.KEY_KP_Tab ||
-            keyval == Gdk.KEY_Up ||
+        if (keyval == Gdk.KEY_Up ||
             keyval == Gdk.KEY_KP_Up ||
             keyval == Gdk.KEY_Down ||
             keyval == Gdk.KEY_KP_Down ||
@@ -149,20 +137,46 @@ const Searchbar = new Lang.Class({
         return (keyval == Gdk.KEY_space);
     },
 
+    _isTabEvent: function(event) {
+        let keyval = event.get_keyval()[1];
+        return (keyval == Gdk.KEY_Tab || keyval == Gdk.KEY_KP_Tab);
+    },
+
     handleEvent: function(event) {
-        if (this._in)
+        // Skip if the search bar is shown and the focus is elsewhere
+        if (this._in && !this._searchEntry.is_focus)
             return false;
 
-        if (this._isKeynavEvent(event))
+        let isEscape = this._isEscapeEvent(event);
+        let isKeynav = this._isKeynavEvent(event);
+        let isTab = this._isTabEvent(event);
+        let isSpace = this._isSpaceEvent(event);
+
+        // Skip these if the search bar is hidden
+        if (!this._in && (isEscape || isKeynav || isTab || isSpace))
             return false;
 
-        if (this._isSpaceEvent(event))
-            return false;
+        // At this point, either the search bar is hidden and the event
+        // is neither escape nor keynav nor space; or the search bar is
+        // shown and has the focus.
+
+        let keyval = event.get_keyval()[1];
+        if (isEscape) {
+            Application.application.change_action_state('search', GLib.Variant.new('b', false));
+            return true;
+        } else if (keyval == Gdk.KEY_Return) {
+            this.emit('activate-result');
+            return true;
+        }
 
         if (!this._searchEntry.get_realized())
             this._searchEntry.realize();
 
-        let handled = false;
+        // Since we can have keynav or space only when the search bar
+        // is shown, we want to handle it. Otherwise it will hinder
+        // text input. However, we don't want to handle tabs so that
+        // focus can be shifted to other widgets.
+        let handled = isKeynav || isSpace;
 
         let preeditChanged = false;
         let preeditChangedId =
@@ -207,62 +221,37 @@ Signals.addSignalMethods(Searchbar.prototype);
 
 const Dropdown = new Lang.Class({
     Name: 'Dropdown',
+    Extends: Gtk.Popover,
 
-    _init: function() {
-        this._sourceView = new Manager.BaseView(Application.sourceManager);
-        this._typeView = new Manager.BaseView(Application.searchTypeManager);
-        this._matchView = new Manager.BaseView(Application.searchMatchManager);
-        // TODO: this is out for now, but should we move it somewhere
-        // else?
-        // this._categoryView = new Manager.BaseView(Application.searchCategoryManager);
+    _init: function(relativeTo) {
+        this.parent({ relative_to: relativeTo, position: Gtk.PositionType.BOTTOM });
 
-        this._sourceView.connect('item-activated',
-                                 Lang.bind(this, this._onItemActivated));
-        this._typeView.connect('item-activated',
-                               Lang.bind(this, this._onItemActivated));
-        this._matchView.connect('item-activated',
-                                Lang.bind(this, this._onItemActivated));
+        let grid = new Gtk.Grid({ orientation: Gtk.Orientation.HORIZONTAL,
+                                  row_homogeneous: true });
+        this.add(grid);
 
-        let frame = new Gtk.Frame({ shadow_type: Gtk.ShadowType.IN,
-                                    opacity: 0.9 });
-        frame.get_style_context().add_class('documents-dropdown');
+        [Application.sourceManager,
+         Application.searchTypeManager,
+         Application.searchMatchManager].forEach(Lang.bind(this, function(manager) {
+             let model = new Manager.BaseModel(manager);
 
-        this.widget = new Gtk.Revealer({ halign: Gtk.Align.CENTER,
-                                         valign: Gtk.Align.START });
-        this.widget.add(frame);
-
-        this._grid = new Gtk.Grid({ orientation: Gtk.Orientation.HORIZONTAL });
-        frame.add(this._grid);
-
-        this._grid.add(this._sourceView.widget);
-        this._grid.add(this._typeView.widget);
-        this._grid.add(this._matchView.widget);
-        //this._grid.add(this._categoryView.widget);
-
-        this.hide();
-        this.widget.show_all();
-    },
-
-    _onItemActivated: function() {
-        this.emit('item-activated');
-    },
-
-    show: function() {
-        this.widget.reveal_child = true;
-    },
-
-    hide: function() {
-        this.widget.reveal_child = false;
+             // HACK: see https://bugzilla.gnome.org/show_bug.cgi?id=733977
+             let popover = new Gtk.Popover();
+             popover.bind_model(model.model, 'app');
+             let w = popover.get_child();
+             w.reparent(grid);
+             w.valign = Gtk.Align.START;
+             w.vexpand = true;
+             popover.destroy();
+         }));
     }
 });
-Signals.addSignalMethods(Dropdown.prototype);
 
 const OverviewSearchbar = new Lang.Class({
     Name: 'OverviewSearchbar',
     Extends: Searchbar,
 
-    _init: function(dropdown) {
-        this._dropdown = dropdown;
+    _init: function() {
         this._selectAll = Application.application.lookup_action('select-all');
 
         this.parent();
@@ -301,7 +290,7 @@ const OverviewSearchbar = new Lang.Class({
 
         this._searchEntry.connect('destroy', Lang.bind(this,
             function() {
-                this._dropdown.widget.reveal_child = false;
+                this._dropdown.hide();
                 Application.searchController.disconnect(this._searchChangedId);
             }));
 
@@ -313,9 +302,12 @@ const OverviewSearchbar = new Lang.Class({
         this._dropdownButton.connect('toggled', Lang.bind(this,
             function() {
                 let active = this._dropdownButton.get_active();
-                this._dropdown.widget.reveal_child = active;
+                if(active)
+                    this._dropdown.show_all();
             }));
-        this._dropdown.connect('item-activated', Lang.bind(this,
+
+        this._dropdown = new Dropdown(this._dropdownButton);
+        this._dropdown.connect('closed', Lang.bind(this,
             function() {
                 this._dropdownButton.set_active(false);
             }));
